# Autogenerated code. DO NOT EDIT.
import py_gen_ml as pgml


class Optimizer(pgml.YamlBaseModel):
    """Optimizer configuration"""

    type: str = "sgd"
    """Optimizer type"""

    learning_rate: float = 0.01
    """Learning rate"""
