# Autogenerated code. DO NOT EDIT.
import enum

import py_gen_ml as pgml


class Activation(str, enum.Enum):
    """Activation function"""

    RELU = "RELU"
    """Rectified Linear Unit"""

    GELU = "GELU"
    """Gaussian Error Linear Unit"""


class Linear(pgml.YamlBaseModel):
    """Linear layer"""

    in_features: int
    """Number of input features"""

    out_features: int
    """Number of output features"""

    activation: Activation = Activation.GELU
    """Activation function"""
