# Autogenerated code. DO NOT EDIT.
import typing

import pydantic
import typer

import py_gen_ml as pgml


class ExplicitCLIArgTestArgs(pgml.YamlBaseModel):
    foo: typing.Annotated[
        typing.Optional[str],
        typer.Option(help='Maps to bar'),
        pydantic.Field(None),
        pgml.ArgRef('bar'),
    ]


class ImplicitCLIArgTestArgs(pgml.YamlBaseModel):
    bar: typing.Annotated[
        typing.Optional[str],
        typer.Option(help='Maps to bar'),
        pydantic.Field(None),
        pgml.ArgRef('bar'),
    ]
