# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: extensions.proto
# Protobuf Python Version: 5.27.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    27,
    2,
    '',
    'extensions.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import descriptor_pb2 as google_dot_protobuf_dot_descriptor__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x10\x65xtensions.proto\x12\x04pgml\x1a google/protobuf/descriptor.proto\"!\n\x03\x41rg\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0c\n\x04path\x18\x02 \x01(\t\"-\n\x03\x43LI\x12\x16\n\x03\x61rg\x18\x01 \x03(\x0b\x32\t.pgml.Arg\x12\x0e\n\x06\x65nable\x18\x02 \x01(\x08\"\xbb\x02\n\rFieldDefaults\x12\x0f\n\x05\x66loat\x18\x01 \x01(\x02H\x00\x12\x10\n\x06\x64ouble\x18\x02 \x01(\x01H\x00\x12\x0f\n\x05int32\x18\x03 \x01(\x05H\x00\x12\x0f\n\x05int64\x18\x04 \x01(\x03H\x00\x12\x10\n\x06uint32\x18\x05 \x01(\rH\x00\x12\x10\n\x06uint64\x18\x06 \x01(\x04H\x00\x12\x10\n\x06sint32\x18\x07 \x01(\x11H\x00\x12\x10\n\x06sint64\x18\x08 \x01(\x12H\x00\x12\x11\n\x07\x66ixed32\x18\t \x01(\x07H\x00\x12\x11\n\x07\x66ixed64\x18\n \x01(\x06H\x00\x12\x12\n\x08sfixed32\x18\x0b \x01(\x0fH\x00\x12\x12\n\x08sfixed64\x18\x0c \x01(\x10H\x00\x12\x0e\n\x04\x62ool\x18\r \x01(\x08H\x00\x12\x10\n\x06string\x18\x0e \x01(\tH\x00\x12\x0f\n\x05\x62ytes\x18\x0f \x01(\x0cH\x00\x12\x0e\n\x04\x65num\x18\x10 \x01(\tH\x00\x42\x06\n\x04typeJ\x04\x08\x12\x10\x15:0\n\x06\x65nable\x12\x1c.google.protobuf.FileOptions\x18\xc3\x08 \x01(\x08\x88\x01\x01:1\n\x07\x66\x61\x63tory\x12\x1f.google.protobuf.MessageOptions\x18\xc3\x08 \x01(\t:8\n\x03\x63li\x12\x1f.google.protobuf.MessageOptions\x18\xc4\x08 \x01(\x0b\x32\t.pgml.CLI:G\n\x07\x64\x65\x66\x61ult\x12\x1d.google.protobuf.FieldOptions\x18\x93\t \x01(\x0b\x32\x13.pgml.FieldDefaults\x88\x01\x01:2\n\nas_varargs\x12\x1d.google.protobuf.FieldOptions\x18\x94\t \x01(\x08\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'extensions_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_ARG']._serialized_start=60
  _globals['_ARG']._serialized_end=93
  _globals['_CLI']._serialized_start=95
  _globals['_CLI']._serialized_end=140
  _globals['_FIELDDEFAULTS']._serialized_start=143
  _globals['_FIELDDEFAULTS']._serialized_end=458
# @@protoc_insertion_point(module_scope)
