# Autogenerated code. DO NOT EDIT.
import enum

import py_gen_ml as pgml


class Activation(str, enum.Enum):
    """Activation function"""

    RELU = "RELU"
    """ReLU activation"""

    GELU = "GELU"
    """Gelu activation"""


class MLP(pgml.YamlBaseModel):
    """MLP configuration"""

    activation: Activation
    """Activation function"""

    num_layers: int
    """Number of layers"""
