# Autogenerated code. DO NOT EDIT.
import typing
import py_gen_ml as pgml

from . import enum_demo_base as base


class MLPPatch(pgml.YamlBaseModel):
    """MLP configuration"""

    activation: typing.Optional[base.Activation] = None
    """Activation function"""

    num_layers: typing.Optional[int] = None
    """Number of layers"""
