# Autogenerated code. DO NOT EDIT.
import typing
import py_gen_ml as pgml

from . import default_enum_base as base


class LinearPatch(pgml.YamlBaseModel):
    """Linear layer"""

    in_features: typing.Optional[int] = None
    """Number of input features"""

    out_features: typing.Optional[int] = None
    """Number of output features"""

    activation: typing.Optional[base.Activation] = None
    """Activation function"""
