# Autogenerated code. DO NOT EDIT.
import py_gen_ml as pgml


class DataLoaderConfig(pgml.YamlBaseModel):
    """DataLoader configuration"""

    batch_size: int
    """Batch size"""

    num_workers: int
    """Number of workers"""

    pin_memory: bool
    """Pin memory"""

    persistent_workers: bool
    """Persistent workers"""

    prefetch_factor: int
    """Prefetch factor"""
