from typing import Final

BASE_MODEL_ALIAS: Final[str] = 'base'
SWEEP_MODEL_ALIAS: Final[str] = 'sweep'
PATCH_MODEL_ALIAS: Final[str] = 'patch'
CLI_ARGS_ALIAS: Final[str] = 'cli_args'
PGML_ALIAS: Final[str] = 'pgml'
