# Autogenerated code. DO NOT EDIT.
import typing
import py_gen_ml as pgml


class OptimizerPatch(pgml.YamlBaseModel):
    """Optimizer configuration"""

    type: typing.Optional[str] = None
    """Optimizer type"""

    learning_rate: typing.Optional[float] = None
    """Learning rate"""
