# Autogenerated code. DO NOT EDIT.
import typing
import py_gen_ml as pgml


class DataLoaderConfigPatch(pgml.YamlBaseModel):
    """DataLoader configuration"""

    batch_size: typing.Optional[int] = None
    """Batch size"""

    num_workers: typing.Optional[int] = None
    """Number of workers"""

    pin_memory: typing.Optional[bool] = None
    """Pin memory"""

    persistent_workers: typing.Optional[bool] = None
    """Persistent workers"""

    prefetch_factor: typing.Optional[int] = None
    """Prefetch factor"""
