# Autogenerated code. DO NOT EDIT.
import typing
import py_gen_ml as pgml


class LinearBlock(pgml.YamlBaseModel):
    """Linear block configuration"""

    num_units: int
    """Number of units"""

    activation: str
    """Activation function"""


class Optimizer(pgml.YamlBaseModel):
    """Optimizer configuration"""

    type: str
    """Type of optimizer"""

    learning_rate: float
    """Learning rate"""


class MLP(pgml.YamlBaseModel):
    """Multi-layer perceptron configuration"""

    layers: typing.List[LinearBlock]
    """List of linear blocks"""


class Training(pgml.YamlBaseModel):
    """Training configuration"""

    mlp: MLP
    """Multi-layer perceptron configuration"""

    optimizer: Optimizer
    """Optimizer configuration"""
